--- llcommon/llsys.cpp.orig	Thu Aug  2 23:13:06 2007
+++ llcommon/llsys.cpp	Thu Aug 16 09:44:08 2007
@@ -46,6 +46,15 @@
 #elif LL_DARWIN
 #	include <sys/sysctl.h>
 #	include <sys/utsname.h>
+#elif LL_FREEBSD
+#	include <fcntl.h>
+#	include <kvm.h>
+#	include <paths.h>
+#	include <unistd.h>
+#	include <sys/param.h>
+#	include <sys/sysctl.h>
+#	include <sys/utsname.h>
+#	include <sys/user.h>
 #elif LL_LINUX
 #	include <sys/utsname.h>
 #	include <unistd.h>
@@ -272,6 +281,14 @@
 		numRead += sscanf(memp, "%*s %u", &virtual_size);
 	}
 	fclose(status_filep);
+#elif LL_FREEBSD
+	kvm_t *kd = kvm_open(NULL, _PATH_DEVNULL, NULL, O_RDONLY, "kvm_open");
+	if (!kd) return 0;
+
+	struct kinfo_proc *process_info; int count;
+	process_info = kvm_getprocs(kd, KERN_PROC_PID, (int)getpid(), &count);
+	if (process_info) virtual_size = process_info->ki_size / 1024;
+	kvm_close(kd);
 #elif LL_SOLARIS
 	char proc_ps[LL_MAX_PATH];
 	sprintf(proc_ps, "/proc/%d/psinfo", (int)getpid());
@@ -318,6 +335,15 @@
 		}
 		fclose(status_filep);
 	}
+#elif LL_FREEBSD
+	kvm_t *kd = kvm_open(NULL, _PATH_DEVNULL, NULL, O_RDONLY, "kvm_open");
+	if (!kd) return 0;
+	
+	struct kinfo_proc *process_info; int count;
+	process_info = kvm_getprocs(kd, KERN_PROC_PID, (int)getpid(), &count);
+	if (process_info) resident_size = process_info->ki_rssize;
+	resident_size *= getpagesize() / 1024;
+	kvm_close(kd);
 #elif LL_SOLARIS
 	char proc_ps[LL_MAX_PATH];
 	sprintf(proc_ps, "/proc/%d/psinfo", (int)getpid());
@@ -361,6 +387,11 @@
 	}
 	mCPUString = out.str();
 	
+#elif LL_FREEBSD
+	char buffer[1024]; size_t size = sizeof(buffer);
+	if(sysctlbyname("hw.model", buffer, &size, NULL, 0) == 0)
+		mCPUString = buffer;
+
 #elif LL_LINUX
 	std::map< LLString, LLString > cpuinfo;
 	FILE* cpuinfo_fp = LLFile::fopen(CPUINFO_FILE, "rb");
@@ -437,7 +468,7 @@
 
 void LLCPUInfo::stream(std::ostream& s) const
 {
-#if LL_WINDOWS || LL_DARWIN || LL_SOLARIS
+#if LL_WINDOWS || LL_DARWIN || LL_FREEBSD || LL_SOLARIS
 	// gather machine information.
 	char proc_buf[CPUINFO_BUFFER_SIZE];		/* Flawfinder: ignore */
 	CProcessor proc;
@@ -491,7 +522,7 @@
 
 	return (U32)state.dwTotalPhys;
 
-#elif LL_DARWIN
+#elif LL_DARWIN || LL_FREEBSD
 	// This might work on Linux as well.  Someone check...
 	unsigned int phys = 0;
 	int mib[2] = { CTL_HW, HW_PHYSMEM };
@@ -525,12 +556,16 @@
 	s << "Avail page Kb:      " << (U32)state.dwAvailPageFile/1024 << std::endl;
 	s << "Total Virtual Kb:   " << (U32)state.dwTotalVirtual/1024 << std::endl;
 	s << "Avail Virtual Kb:   " << (U32)state.dwAvailVirtual/1024 << std::endl;
-#elif LL_DARWIN
+#elif LL_DARWIN || LL_FREEBSD
 	U64 phys = 0;
 
 	size_t len = sizeof(phys);	
 	
+#if LL_DARWIN
 	if(sysctlbyname("hw.memsize", &phys, &len, NULL, 0) == 0)
+#else
+	if(sysctlbyname("hw.physmem", &phys, &len, NULL, 0) == 0)
+#endif
 	{
 		s << "Total Physical Kb:  " << phys/1024 << std::endl;
 	}
